@@ -1,17 +1,16 @@
-import os
 import re
 import json
 import gcsfs
 
-from google.cloud import aiplatform, pubsub_v1
+from google.cloud import aiplatform
 
 from captur_ml_sdk.dtypes.exceptions import (
     ModelNotFoundError,
+    ModelHasNoLiveVersionsError,
     VersionNotFoundError,
-    ModelHasNoLiveVersionsError
+    VersionNotLiveError
 )
 
-
 def get_model_list_from_aiplatform():
     """Returns list of trained models with details"""
     api_endpoint = "us-central1-aiplatform.googleapis.com"
@@ -34,19 +33,41 @@ def read_json_from_gcp_storage(file_path, project="capturpwa"):
         data (dict) : data on model names and versions
             - format:
     {
-        "<model1_name>": [
-            "<model_version_1_id>",
-            "<model_version_2_id>",
-            "<model_version_3_id>"
-        ],
-        "<model2_name>": [
-            "<model_version_1_id>"
-        ],
-        "<model3_name>": []
+        "<model1_name>": {
+            "type": "classification",
+            "model_versions": [
+                {
+                    "model_id": "<model_version_1_id>",
+                    "dataset_id": "<dataset1_id>"
+                },
+                {
+                    "model_id": "<model_version_2_id>",
+                    "dataset_id": "<dataset2_id>"
+                },
+                {
+                    "model_id": "<model_version_3_id>",
+                    "dataset_id": "<dataset3_id>"
+                },
+            ]
+        }
+        "<model2_name>": {
+            "type": "object_detection",
+            "model_versions": [
+                {
+                    "model_id": "<model_version_1_id>",
+                    "dataset_id": "<dataset1_id>"
+                },
+                {
+                    "model_id": "<model_version_2_id>",
+                    "dataset_id": "<dataset2_id>"
+                }
+            ]
+        }
+        "<model3_name>": {}
     }
     ----------------
     """
-    gcs_file_system = gcsfs.GCSFileSystem("capturpwa")
+    gcs_file_system = gcsfs.GCSFileSystem(project)
     with gcs_file_system.open(file_path, "r") as f:
         data = json.loads(f.read())
     return data
@@ -90,16 +111,15 @@ def get_all_available_models(with_filter=None):
     return all_model_dicts
 
 
-def get_model_from_specific_version(model_name, model_version):
+def check_specific_version_exists(model_name, model_version):
     """
-    Returns a model information dictionary based on a specified
-    model name and specific model version.
+    Checks whether a specific version of a model exists within GCP.
     ----------------
     args:
         model_name (str) : display name of the model
         model_version (str) : specific model version e.g. 4837405867162748509
     returns:
-        res (list[dict]) : model information list
+        specific_version_exists (bool)
     ----------------
     """
     filters = {
@@ -108,16 +128,12 @@ def get_model_from_specific_version(model_name, model_version):
     }
     res = get_all_available_models(with_filter=filters)
 
-    if len(res) == 0:
-        return None
+    specific_version_exists = False
 
-    # res is a list but since we want a specific version, it will only have a single item
-    #
-    # That will a dictionary, where "name" is the URL to the model,
-    # e.g. projects/73629791501/locations/us-central1/models/91098409123371872483
-    #
-    # We only care about the ID at the end, which act as a UUID to our model on GCP
-    return res[0]["name"].split("/")[-1]
+    if len(res) > 0:
+        specific_version_exists = True
+
+    return specific_version_exists
 
 
 def get_list_of_versions_from_gcp(model_name):
@@ -132,7 +148,47 @@ def get_list_of_versions_from_gcp(model_name):
     """
     model_data = read_json_from_gcp_storage(
         file_path="captur-ml/models/model_bank.json")
-    return model_data.get(model_name, [])
+    if model_data.get(model_name):
+        model_version_list = model_data.get(model_name).get("model_versions", [])
+    else:
+        model_version_list = []
+    return model_version_list
+
+
+def check_specific_version_is_live(live_versions, model_version):
+    """
+    Checks whether a specific version of a model is live.
+    ----------------
+    args:
+        live_versions (list) : list of live versions from model_bank.json
+        model_version (str) : specific model version e.g. 4837405867162748509
+    returns:
+        version_is_live (bool)
+    ----------------
+    """
+    version_is_live = False
+    for version in live_versions:
+        if version.get("model_id") == model_version:
+            version_is_live = True
+    return version_is_live
+
+
+def get_model_type_from_gcp(model_name):
+    """
+    Returns the type of a model.
+    ----------------
+    args:
+        model_name (str) : display name of the model
+    returns:
+        model_type (str) : type of model, (currently only {"classification", "object_detection"})
+    ----------------
+    """
+    model_data = read_json_from_gcp_storage(
+        file_path="captur-ml/models/model_bank.json")
+    try:
+        return model_data.get(model_name).get("type")
+    except AttributeError:
+        raise ModelNotFoundError(f"{model_name} not found.")
 
 
 def get_model_from_latest(model_version, list_of_model_versions):
@@ -163,7 +219,7 @@ def get_model_from_latest(model_version, list_of_model_versions):
     if model_countback > len(list_of_model_versions):
         return None
 
-    return list_of_model_versions[-model_countback]
+    return list_of_model_versions[-model_countback].get("model_id")
 
 
 def get_model_from_absolute_version(model_version, list_of_model_versions):
@@ -191,7 +247,7 @@ def get_model_from_absolute_version(model_version, list_of_model_versions):
     if model_version >= len(list_of_model_versions):
         return None
 
-    return list_of_model_versions[model_version]
+    return list_of_model_versions[model_version].get("model_id")
 
 
 def get_requested_model_id(model_name: str, model_version: str):
@@ -207,27 +263,19 @@ def get_requested_model_id(model_name: str, model_version: str):
             "HEAD~1"
 
     returns:
-        model information (dict) or False
+        model version with model_id and dataset_id (dict) or False
 
     raises:
         ModelNotFoundError: raised if this model_name is not found
         VersionNotFoundError: raised if the model_name is found but the specified model_version is not
         ModelHasNoLiveVersionsError: raised if model_name exists but no live versions are present in the model bank file
+        VersionNotLiveError: raised if model_name exists but version is not live (in the model bank)
     ----------------
     """
 
-    # TODO: avoid running this twice by e.g. wrapping most functions in this file in a class
     if not get_all_available_models(with_filter={"display_name": model_name}):
         raise ModelNotFoundError(f"Model: {model_name} not found.")
 
-    if re.match("[0-9]{19}", model_version):
-        model_id = get_model_from_specific_version(model_name, model_version)[
-            0]["name"].split("/")[-1]
-
-        if model_id is None:
-            raise VersionNotFoundError(
-                f"Model {model_name} does not have a version {model_version}")
-
     model_version_data = get_list_of_versions_from_gcp(model_name)
 
     if len(model_version_data) == 0:
@@ -236,23 +284,33 @@ def get_requested_model_id(model_name: str, model_version: str):
             f"Please specifiy the version directly or deploy a live version."
         )
 
-    absolute_model_version = None
+    if re.match("[0-9]{19}", model_version):
+        model_id = model_version
+        if not check_specific_version_is_live(model_version_data, model_id):
+            raise VersionNotLiveError(
+                f"{model_id} not found under {model_name} in model bank."
+            )
 
     if re.match("^HEAD(~[1-9][0-9]*)?$", model_version):
-        absolute_model_version = get_model_from_latest(
+        model_id = get_model_from_latest(
             model_version, model_version_data)
 
     if re.match("^v[1-9][0-9]*$", model_version):
-        absolute_model_version = get_model_from_absolute_version(
+        model_id = get_model_from_absolute_version(
             model_version, model_version_data)
 
-    if absolute_model_version is not None:
-        return get_model_from_specific_version(model_name, absolute_model_version)
+    if model_id is None:
+        raise VersionNotFoundError(
+            f"Model {model_name} cannot get version {model_version}"
+            f"because there are only {len(model_version_data)} live versions."
+        )
 
-    raise VersionNotFoundError(
-        f"Model {model_name} cannot get version {model_version}"
-        f"because there are only {len(model_version_data)} live versions."
-    )
+    if not check_specific_version_exists(model_name, model_id):
+        raise VersionNotFoundError(
+            f"Model {model_name} exists but {model_id} not found."
+        )
+
+    return model_id
 
 
 def check_file_exists(filepath):
@@ -275,4 +333,4 @@ def get_dataset_id_from_model_id(model_id):
     """
     model_data = read_json_from_gcp_storage(
         file_path="captur-ml/models/model_dataset_mapping.json")
-    return model_data.get(model_id)
\ No newline at end of file
+    return model_data.get(model_id)
